! ***************************************************************************
! *                                                                         *
! *   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
! *   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
! *                                                                         *     
! *   This program is free software; you can redistribute it and/or modify  *
! *   it under the terms of the GNU General Public License version 2 as     *
! *   published by the Free Software Foundation.                            *
! *                                                                         *
! ***************************************************************************
! ### writeDisk4dd ### 

!     Writes orbitals, potenials, Lagrange multipliers (diagonal 
!     and off-diagonal) and multipole expansion coefficients to a disk 
!     file in either formatted or unformatted form

subroutine writeDisk4dd (psi,pot,wk0,wk1,wk2,wk3)
  use params
  use discret
  use scf
  use commons8

  implicit none
  character*80 :: datetime
  integer :: i4,i1beg,iorb,ngrid,li8tmp
  integer*8 :: i8tmp1

  real (PREC), allocatable :: r8mxsize(:)
  real (PREC), dimension(*) :: psi,pot,wk0,wk1,wk2,wk3

  open(9999,file='out4dd.dat', status='unknown',form='formatted')
  
!  call getDateTime(datetime)
  write(9999,'(a80)') header
!  write(9999,'(a80)') datetime
  write(9999,'(" nnu nmu ")') 
  write(9999,formint) nni,nmu(1)
  write(9999,'(" r  rinf")') 
  write(9999,formfp64) r,rinf
  write(9999,'(" z1 z2 ")') 
  write(9999,formfp64) z1,z2
  write(9999,'(" norb nel  ")') 	
  write(9999,formint) norb,nel

  write(9999,'(" ordering of orbitals ")')
  write(9999,'(1x,i3,1x,a8,1x,a1)') (iorn(iorb),bond(iorb),gut(iorb),iorb=1,norb)

  write(9999,'(" orbital energies: ")')  	
  do iorb=1,norb
     write(9999,formfp64) eng(iorb)
  enddo
  
  allocate(r8mxsize(nni*mxnmu))

  do iorb=1,norb

     i1beg=i1b(iorb)
     ngrid=i1si(iorb)

     do i4=1,ngrid
        r8mxsize(i4)=pot(i1beg+i4-1)
     enddo

     write(9999,'(/" orbital=",i3,1x,a8,1x,a1," Coulomb potential" )') iorn(iorb),bond(iorb),gut(iorb)
     call prtmatcw(nni,mxnmu,r8mxsize,9999)

     do i4=1,ngrid
        r8mxsize(i4)=psi(i1beg+i4-1)
     enddo

1000 format(i5,i4,1x,a8,a1)
     write(9999,'(/" orbital=",i3,1x,a8,1x,a1," orbital function")') iorn(iorb),bond(iorb),gut(iorb)
!     write(9999,1000) iorn(iorb),bond(iorb),gut(iorb)
!     write(9999,'(/" orbital=",i3,i3,a8)') iorn(iorb),bond(iorb),gut(iorb)
     call prtmatcw(nni,mxnmu,r8mxsize,9999)

     call putin (nni,mxnmu,isymOrb(iorb),psi(i1beg),wk3)
     call diff1nu (mxnmu,wk3,wk0,wk1,wk2)
     call putout (nni,mxnmu,r8mxsize,wk0)

     write(9999,'(/" orbital=",i3,1x,a8,1x,a1," / nu derivative" )') iorn(iorb),bond(iorb),gut(iorb)
     call prtmatcw(nni,mxnmu,r8mxsize,9999)
     
     call diff1mu (mxnmu,wk3,wk2)
     call putout (nni,mxnmu,r8mxsize,wk2)
     write(9999,'(/" orbital=",i3,1x,a8,1x,a1," / mu derivative" )') iorn(iorb),bond(iorb),gut(iorb)
     call prtmatcw(nni,mxnmu,r8mxsize,9999)

     call diff1mu (mxnmu,wk0,wk2)
     call putout (nni,mxnmu,r8mxsize,wk2)
     write(9999,'(/" orbital=",i3,1x,a8,1x,a1," / nu,mu derivative" )') iorn(iorb),bond(iorb),gut(iorb)
     call prtmatcw(nni,mxnmu,r8mxsize,9999)
  enddo
  
  close(9999)

end subroutine writeDisk4dd

